import os
from collections import Counter, defaultdict
from contextlib import suppress

import numpy as np
from rich import print
from rich.panel import Panel
from rich.prompt import Confirm, Prompt


def filter_array(filter_func, words) -> np.ndarray:
    return np.array(list(filter(filter_func, words)))


def filter_words(words, letters, pos) -> np.ndarray:
    for char in letters:
        if pos == 0:  # not inside
            words = filter_array(lambda x: x.find(char) == -1, words)
        elif pos >= 1 and pos <= 5:  # known position
            words = filter_array(lambda x: x[pos - 1] == char, words)
        elif pos < 0:  # known not position
            words = filter_array(lambda x: x.find(char) != -1, words)
            words = filter_array(lambda x: x[-pos - 1] != char, words)
    return words


def get_words() -> np.ndarray:
    # Get data
    # filepath = os.path.join(os.path.dirname(__file__), "data/popular_filtered.txt")
    filepath = os.path.join(os.path.dirname(__file__), "data/words_filtered.txt")
    return np.loadtxt(filepath, dtype="str")


def print_help() -> None:
    print(
        Panel(
            "Help (?) [bold blue]position[/]: "
            + "([bold green]green[/]): 1 to 5, "
            + "([bold]black[/]): 0, "
            + "([bold yellow]yellow[/]): -1 to -5"
        )
    )


def format_guess(guess) -> str:
    formated_guess = ""
    for char in guess:
        if char.isalpha():
            formated_guess += f"[bold white on green]{char.upper()}[/]"
        else:
            formated_guess += f"{char}"

    return f"{formated_guess}"


def find_most_frequent_letters(words, n=4) -> list:
    # Count letters
    letters: dict = defaultdict(int)
    for word in words:
        for k, v in Counter(word).items():  # type: str, int
            letters[k] += v
    sorted_keys = sorted(letters.items(), key=lambda kv: kv[1], reverse=True)

    return [kv[0] for kv in sorted_keys[:n]]


def find_most_representative_words(words, n=4) -> list:
    most_freq_letters = find_most_frequent_letters(words, n)
    sub_list = []
    for word in words:
        if all(char in word for char in most_freq_letters):
            sub_list.append(word)
    return sub_list[:n]


def cli() -> None:

    # Init by parsing wordlist
    words = get_words()

    # Helper
    print_help()

    guess = "12345"

    with suppress(KeyboardInterrupt):
        while True:
            # Print best guess words
            print(f"Best guess: ({find_most_representative_words(words)})")

            # Get user input to filter words
            raw_input = Prompt.ask(
                f"Guess word ({format_guess(guess)}) "
                + "[bold][[red]<letter(s)>[/red] [blue]<position>[/blue]][/]"
            )
            if raw_input[0] == "?":
                print_help()
                continue
            else:
                # Parse input
                letters, pos_str = raw_input.split(" ")
                pos = int(pos_str)
                # Filter words
                words = filter_words(words, letters, pos)
                if pos > 0:
                    guess = guess[: pos - 1] + letters[0] + guess[pos:]

            # Print status
            if len(words) == 1:
                guess = words[0]
                print(Panel(f"Solution: [bold green]{guess}[/] :partying_face:"))
                break

            elif len(words) == 0:
                print("Failed!", ":sob:")

                restart = Confirm.ask("Restart guess?")
                if restart:
                    words = get_words()
                    guess = "12345"
                else:
                    break

            else:
                print("Possible words:", words)


if __name__ == "__main__":
    cli()



import click
import numpy as np


@click.command()
@click.option("--src", help="File containing raw word list")
@click.option("--size", default=5, help="word length filter size")
def filter_word_list(src, size) -> None:

    # Get all words in english dictionary
    words = np.loadtxt(src, dtype=str)

    # Filter only words with length of 5
    words = words[np.where(np.array([len(word) for word in words]) == size)]

    # # Filter only words with alphabetical characters
    # words = words[np.where(np.array([word.isalpha() for word in words]) is True)]

    # Make all words lowercase
    words = np.array([word.lower() for word in words])

    # Store words
    np.savetxt(src.replace(".txt", "_filtered.txt"), words, fmt="%s")


if __name__ == "__main__":
    filter_word_list()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wordlebee']

package_data = \
{'': ['*'], 'wordlebee': ['data/*']}

install_requires = \
['numpy>=1.22.1,<2.0.0', 'rich>=11.0.0,<12.0.0']

entry_points = \
{'console_scripts': ['wordlebee = wordlebee.__main__:cli'],
 'pipx.run': ['wordlebee = wordlebee.__main__:cli']}

setup_kwargs = {
    'name': 'wordlebee',
    'version': '0.7.0',
    'description': 'wordle word guessing helper bee',
    'long_description': '<div align="center">\n\n<h1>\n    <img width="500" align="center" src="assets/wordlebee-logo.svg">\n</h1>\n\n[![PyPi Version](https://img.shields.io/pypi/v/wordlebee.svg?style=flat-square)](https://pypi.org/project/wordlebee/)\n[![PyPi Python versions](https://img.shields.io/pypi/pyversions/wordlebee.svg?style=flat-square)](https://pypi.org/project/wordlebee/)\n[![License](https://img.shields.io/badge/license-MIT-blue?style=flat-square)](#license)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg?style=flat-square)](#black)\n[![Downloads](https://pepy.tech/badge/wordlebee?style=flat-square)](https://pepy.tech/project/wordlebee)\n\n***A cli wordle word guessing helper bee to solve the wordle puzzle of the day.***\n\n</div>\n\n## Usage\n\n```\npython -m wordlebee\n```\n\n## Installation\n\nInstall `wordlebee` in isolated environment:\n\n```\npipx install wordlebee\n```\n\nInstall `wordlebee`:\n\n```\npip install wordlebee\n```\n\n## Development\n\nInstall conda enviroment:\n\n```\nconda env create -f environment.yml\n```\n\nInstall using `poetry`:\n\n```\npoetry install\n```\n',
    'author': 'Mrlento234',
    'author_email': 'lento.manickathan@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
